@@ -25,8 +25,7 @@
 #include <glib.h>
 
 #ifndef TOTEM_PL_PARSER_MINI
-#include <zlib.h>
-
+#include <libsoup/soup-gnome.h>
 #include "xmlparser.h"
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
@@ -104,10 +103,10 @@ static TotemPlParserResult
 parse_rss_item (TotemPlParser *parser, xml_node_t *parent)
 {
 	const char *title, *uri, *description, *author;
-	const char *pub_date, *duration, *filesize;
+	const char *pub_date, *duration, *filesize, *content_type;
 	xml_node_t *node;
 
-	title = uri = description = author = NULL;
+	title = uri = description = author = content_type = NULL;
 	pub_date = duration = filesize = NULL;
 
 	for (node = parent->child; node != NULL; node = node->next) {
@@ -137,6 +136,7 @@ parse_rss_item (TotemPlParser *parser, xml_node_t *parent)
 			if (tmp != NULL &&
 			    g_str_has_prefix (tmp, "audio/") == FALSE)
 				continue;
+			content_type = tmp;
 
 			tmp = xml_parser_get_property (node, "url");
 			if (tmp != NULL)
@@ -174,6 +174,7 @@ parse_rss_item (TotemPlParser *parser, xml_node_t *parent)
 					 TOTEM_PL_PARSER_FIELD_AUTHOR, author,
 					 TOTEM_PL_PARSER_FIELD_DURATION, duration,
 					 TOTEM_PL_PARSER_FIELD_FILESIZE, filesize,
+					 TOTEM_PL_PARSER_FIELD_CONTENT_TYPE, content_type,
 					 NULL);
 	}
 
@@ -572,22 +573,71 @@ totem_pl_parser_add_xml_feed (TotemPlParser *parser,
 #endif /* !HAVE_GMIME */
 }
 
+static GByteArray *
+totem_pl_parser_load_http_itunes (const char *uri,
+				  gboolean    debug)
+{
+	SoupMessage *msg;
+	SoupSession *session;
+	GByteArray *data;
+
+	if (debug)
+		g_print ("Loading ITMS playlist '%s'\n", uri);
+
+	session = soup_session_sync_new_with_options (
+	    SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_GNOME_FEATURES_2_26,
+	    SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_CONTENT_DECODER,
+	    SOUP_SESSION_USER_AGENT, "iTunes/10.0.0",
+	    SOUP_SESSION_ACCEPT_LANGUAGE_AUTO, TRUE,
+	    NULL);
+
+	msg = soup_message_new (SOUP_METHOD_GET, uri);
+	soup_session_send_message (session, msg);
+	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		data = g_byte_array_new ();
+		g_byte_array_append (data,
+				     (guchar *) msg->response_body->data,
+				     msg->response_body->length);
+	} else {
+		return NULL;
+	}
+	g_object_unref (msg);
+	g_object_unref (session);
+
+	return data;
+}
+
 static const char *
-totem_pl_parser_parse_itms_doc (xml_node_t *item)
+totem_pl_parser_parse_plist (xml_node_t *item)
 {
 	for (item = item->child; item != NULL; item = item->next) {
 		/* What we're looking for looks like:
-		 * <key>feedURL</key><string>URL</string> */
+		 *  <key>action</key>
+		 *  <dict>
+		 *    <key>kind</key><string>Goto</string>
+		 *    <key>url</key><string>URL</string>
+		 */
 		if (g_ascii_strcasecmp (item->name, "key") == 0
-		    && g_ascii_strcasecmp (item->data, "feedURL") == 0
-		    && item->next != NULL) {
-			item = item->next;
-			if (g_ascii_strcasecmp (item->name, "string") == 0)
-				return item->data;
+		    && g_ascii_strcasecmp (item->data, "action") == 0) {
+			xml_node_t *node = item->next;
+
+			if (node && g_ascii_strcasecmp (node->name, "[CDATA]") == 0)
+				node = node->next;
+			if (node && g_ascii_strcasecmp (node->name, "dict") == 0) {
+				for (node = node->child; node != NULL; node = node->next) {
+					if (g_ascii_strcasecmp (node->name, "key") == 0 &&
+					    g_ascii_strcasecmp (node->data, "url") == 0 &&
+					    node->next != NULL) {
+						node = node->next;
+						if (g_ascii_strcasecmp (node->name, "string") == 0)
+							return node->data;
+					}
+				}
+			}
 		} else {
 			const char *ret;
 
-			ret = totem_pl_parser_parse_itms_doc (item);
+			ret = totem_pl_parser_parse_plist (item);
 			if (ret != NULL)
 				return ret;
 		}
@@ -596,101 +646,72 @@ totem_pl_parser_parse_itms_doc (xml_node_t *item)
 	return NULL;
 }
 
+static char *
+totem_pl_parser_parse_html (char *data, gsize len, gboolean debug)
+{
+	char *s, *end;
+
+	s = g_strstr_len (data, len, "feed-url=\"");
+	if (s == NULL)
+		return NULL;
+	s += strlen ("feed-url=\"");
+	if (*s == '\0')
+		return NULL;
+	end = g_strstr_len (s, len - (s - data), "\"");
+	if (end == NULL)
+		return NULL;
+	return g_strndup (s, end - s);
+}
+
 static GFile *
-totem_pl_parser_get_feed_uri (char *data, gsize len)
+totem_pl_parser_get_feed_uri (char *data, gsize len, gboolean debug)
 {
 	xml_node_t* doc;
 	const char *uri;
 	GFile *ret;
+	GByteArray *content;
 
 	uri = NULL;
 
+	/* Probably HTML, look for feed-url */
+	if (g_strstr_len (data, len, "feed-url") != NULL) {
+		char *uri;
+		uri = totem_pl_parser_parse_html (data, len, debug);
+		if (debug)
+			g_print ("Found feed-url in HTML: '%s'\n", uri);
+		if (uri == NULL)
+			return NULL;
+		ret = g_file_new_for_uri (uri);
+		g_free (uri);
+		return ret;
+	}
+
 	doc = totem_pl_parser_parse_xml_relaxed (data, len);
 	if (doc == NULL)
 		return NULL;
 
 	/* If the document has no name */
 	if (doc->name == NULL
-	    || g_ascii_strcasecmp (doc->name , "Document") != 0) {
+	    || g_ascii_strcasecmp (doc->name, "plist") != 0) {
 		xml_parser_free_tree (doc);
 		return NULL;
 	}
 
-	uri = totem_pl_parser_parse_itms_doc (doc);
+	/* Redirect plist? Find a goto action */
+	uri = totem_pl_parser_parse_plist (doc);
+
+	if (debug)
+		g_print ("Found redirect URL: %s\n", uri);
+
 	if (uri == NULL) {
-		xml_parser_free_tree (doc);
 		return NULL;
+	} else {
+
+		content = totem_pl_parser_load_http_itunes (uri, debug);
+		ret = totem_pl_parser_get_feed_uri ((char *) content->data, content->len, debug);
+		g_byte_array_free (content, TRUE);
 	}
 
-	ret = g_file_new_for_uri (uri);
-	xml_parser_free_tree (doc);
-
-	return ret;
-}
-
-static GByteArray *
-totem_pl_parser_get_content_decompressed (GFile *file)
-{
-	GFileInputStream *file_in;
-	GInputStream *converter_in;
-	GConverter *decompressor;
-	GByteArray *data;
-	gsize position;
-	gssize n_read;
-
-	file_in = g_file_read (file, NULL, NULL);
-	if (file_in == NULL)
-		return NULL;
-
-	decompressor = G_CONVERTER (g_zlib_decompressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP));
-	converter_in = g_converter_input_stream_new (G_INPUT_STREAM (file_in), decompressor);
-	g_object_unref (file_in);
-	g_object_unref (decompressor);
-
-#define BLOCK_SIZE 8192
-	position = 0;
-	data = g_byte_array_sized_new (BLOCK_SIZE + 1);
-	while ((n_read = g_input_stream_read (converter_in,
-					      data->data + position,
-					      BLOCK_SIZE,
-					      NULL, NULL)) > 0) {
-		position += n_read;
-		g_byte_array_set_size (data, position + BLOCK_SIZE + 1);
-	}
-
-	g_object_unref (converter_in);
-
-	if (n_read < 0) {
-		g_byte_array_free (data, TRUE);
-		return NULL;
-	}
-
-	return data;
-}
-
-static char *
-totem_pl_parser_get_itms_uri (const char *data,
-			      gsize data_len)
-{
-	char *s, *end, *ret;
-#define ITMS_OPEN "<body onload=\"return itmsOpen('"
-
-	/* The bit of text looks like:
-	 * <body onload="return itmsOpen('itms://ax.phobos.apple.com.edgesuite.net/WebObjects/MZStore.woa/wa/viewPodcast?id=207870198&amp;ign-mscache=1','http://www.apple.com/uk/itunes/affiliates/download/?itmsUrl=itms%3A%2F%2Fax.phobos.apple.com.edgesuite.net%2FWebObjects%2FMZStore.woa%2Fwa%2FviewPodcast%3Fid%3D207870198%26ign-mscache%3D1','userOverridePanel',false)"> */
-
-	s = g_strstr_len (data, data_len, ITMS_OPEN);
-	if (s == NULL)
-		return NULL;
-	s += strlen (ITMS_OPEN);
-	if (s[0] != 'i' || s[1] != 't' || s[2] != 'm' || s[3] != 's')
-		return NULL;
-
-	end = g_strstr_len (s, data + data_len - s, "\'");
-	if (end == NULL)
-		return NULL;
-
-	ret = g_strndup (s, end - s);
-	memcpy (ret, "http", 4);
 	return ret;
 }
 
@@ -706,39 +727,24 @@ totem_pl_parser_add_itms (TotemPlParser *parser,
 #else
 	GByteArray *content;
 	char *itms_uri;
-	GFile *itms_file, *feed_file;
+	GFile *feed_file;
 	TotemPlParserResult ret;
 
-	if (g_file_has_uri_scheme (file, "itms") == FALSE) {
-		content = totem_pl_parser_get_content_decompressed (file);
-		if (content == NULL)
-			return TOTEM_PL_PARSER_RESULT_ERROR;
-
-		/* Look for the link to the itms on phobos */
-		itms_uri = totem_pl_parser_get_itms_uri ((const char*) content->data, content->len);
-		g_byte_array_free (content, TRUE);
-	} else {
+	if (g_file_has_uri_scheme (file, "itms") != FALSE) {
 		itms_uri= g_file_get_uri (file);
 		memcpy (itms_uri, "http", 4);
-	}
-
-	if (itms_uri == NULL)
-		return TOTEM_PL_PARSER_RESULT_ERROR;
-
-	/* Get the phobos linked, in some weird iTunes only format */
-	itms_file = g_file_new_for_uri (itms_uri);
-	g_free (itms_uri);
-
-	content = totem_pl_parser_get_content_decompressed (itms_file);
-	if (content == NULL) {
-		DEBUG(itms_file, g_print ("Couldn't load contents for itms_file %s\n", uri));
-		g_object_unref (itms_file);
+	} else if (g_file_has_uri_scheme (file, "http") != FALSE) {
+		itms_uri = g_file_get_uri (file);
+	} else {
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
-	g_object_unref (itms_file);
+
+	/* Load the file using iTunes user-agent */
+	content = totem_pl_parser_load_http_itunes (itms_uri, totem_pl_parser_is_debugging_enabled (parser));
 
 	/* And look in the file for the feedURL */
-	feed_file = totem_pl_parser_get_feed_uri ((char *) content->data, content->len);
+	feed_file = totem_pl_parser_get_feed_uri ((char *) content->data, content->len,
+						  totem_pl_parser_is_debugging_enabled (parser));
 	g_byte_array_free (content, TRUE);
 	if (feed_file == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
@@ -761,27 +767,16 @@ totem_pl_parser_is_itms_feed (GFile *file)
 
 	uri = g_file_get_uri (file);
 
-	if (g_file_has_uri_scheme (file, "itms") != FALSE) {
-		if (strstr (uri, "phobos.apple.com") != NULL ||
-		    strstr (uri, "itunes.apple.com") != NULL) {
-			if (strstr (uri, "viewPodcast") != NULL) {
-				g_free (uri);
-				return TRUE;
-			}
+	if (g_file_has_uri_scheme (file, "itms") != FALSE ||
+	    (g_file_has_uri_scheme (file, "http") != FALSE &&
+	     strstr (uri, ".apple.com/") != FALSE)) {
+		if (strstr (uri, "/podcast/") != NULL ||
+		    strstr (uri, "viewPodcast") != NULL) {
+			g_free (uri);
+			return TRUE;
 		}
 	}
 
-	if (strstr (uri, "phobos.apple.com/") != NULL
-	    && strstr (uri, "viewPodcast") != NULL) {
-		g_free (uri);
-		return TRUE;
-	}
-
-	if (strstr (uri, "itunes.com/podcast") != NULL) {
-		g_free (uri);
-		return TRUE;
-	}
-
 	g_free (uri);
 
 	return FALSE;
