@@ -559,10 +559,14 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     "String representing the identifier for an entry", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
-	pspec = g_param_spec_string ("subtitle-uri", "uri",
+	pspec = g_param_spec_string ("subtitle-uri", "subtitle-uri",
 				     "Subtitle URI to be added", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("content-type", "content-type",
+				     "Content type for the video stream", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 }
 
 static void
